#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
@author: HJK
@file: __init__.py
@time: 2019-02-08
"""

import click

from .config import Options
from .glovar import Glovar
from .train import TrainTable


@click.command()
@click.version_option()
@click.option("-f", "--from-station", prompt="请输入出发地", help="出发地")
@click.option("-t", "--to-station", prompt="请输入目的地", help="目的地")
@click.option("-d", "--date", prompt="请输入日期（YYYY-MM-DD）", help="日期")
@click.option("-s", "--seats", help="限制座位")
@click.option("-n", "--train-no", help="限制车次")
@click.option("-z", "--zmode", default=False, is_flag=True, help="高级模式，查询中间站点")
@click.option("-zz", "--zzmode", default=False, is_flag=True, help="终极模式，查询所有中间站点")
@click.option("-r", "--remaining", default=False, is_flag=True, help="只看有票")
@click.option("--gcd", default=False, is_flag=True, help="只看高铁动车城际")
@click.option("--ktz", default=False, is_flag=True, help="只看普快特快直达等")
@click.option("--proxies-file", help="代理列表文件")
@click.option("--stations-file", help="站点信息文件")
@click.option("--cdn-file", help="CDN文件")
def main(
    from_station,
    to_station,
    date,
    seats,
    train_no,
    zmode,
    zzmode,
    remaining,
    gcd,
    ktz,
    proxies_file,
    stations_file,
    cdn_file,
):
    """
    12306查票助手 https://github.com/0xHJK/x12306

    Example：python x12306.py -f 上海 -t 北京 -d "2019-03-01" -n "G16 G18 G22" -r

    如果查询失败的话，请修改glovar.py中的QUERY_URL和JSESSIONID
    """
    opts = Options()
    opts.fs = from_station
    opts.ts = to_station
    if date:
        opts.date = date
    if seats:
        opts.seats = seats
    if train_no:
        opts.train_no = train_no
    opts.zmode = zmode
    opts.zzmode = zzmode
    opts.gcd = gcd
    opts.ktz = ktz
    opts.remaining = remaining
    if proxies_file:
        opts.proxies_file = proxies_file
    if stations_file:
        opts.stations_file = stations_file
    if cdn_file:
        opts.cdn_file = cdn_file

    print("\n-----------------------")
    print(opts)
    tt = TrainTable()
    tt.update_trains()
    tt.echo()



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
@author: HJK
@file: train.py
@time: 2019-02-08
"""
"""
    搜索结果中的班次信息
"""

import time
import re
import prettytable as pt

from .glovar import Glovar, QUERY_URL
from .request import Request
from .utils import code_to_station, station_to_code, colorize


class Train:
    """
    搜索结果中的班次信息，包括车次号、余票、时间等信息
    """

    def __init__(self):
        self.full_no = ""  # 编号全称
        self.no = ""  # 编号简称
        self._fs_code = ""
        self._ts_code = ""
        self._fs = ""
        self._ts = ""
        self.start_time = ""
        self.end_time = ""
        self.duration = ""
        self.remaining = []
        # 途经的车站（站编码，站名，到达时间）元组
        self.stations = []

    def __str__(self):
        colored_remaining = [
            colorize(s, "green") if s != "无" and s != "--" else s
            for s in self.remaining
        ]
        remaining_seats = "/".join(colored_remaining)
        if re.match("[GCD]", self.no):
            colored_no = colorize(self.no, self.no[0].lower())
        else:
            colored_no = colorize(self.no, "o")
        s = "{:^5} | {:^5} | {:\u3000<4} | {:\u3000<4} | {:^5} | {} | {:^5}".format(
            colored_no,
            self.start_time,
            self._fs,
            self._ts,
            self.end_time,
            remaining_seats,
            self.duration,
        )
        return s

    def __eq__(self, other):
        """ 车次、出发站和目的地相同判断为相等 """
        return (
            self.full_no == other.full_no
            and self._fs == other._fs
            and self._ts == other._ts
        )

    def __lt__(self, other):
        """ 根据车次号排序 """
        return self.start_time < other.start_time

    def __gt__(self, other):
        """ 根据车次号排序 """
        return self.start_time > other.start_time

    @property
    def fs_code(self):
        return self._fs_code

    @fs_code.setter
    def fs_code(self, fs_code):
        self._fs_code = fs_code
        self._fs = code_to_station(fs_code)

    @property
    def ts_code(self):
        return self._ts_code

    @ts_code.setter
    def ts_code(self, ts_code):
        self._ts_code = ts_code
        self._ts = code_to_station(ts_code)

    @property
    def has_remaining(self) -> bool:
        for r in self.remaining:
            if r != "无" and r != "--":
                return True
        return False

    @property
    def row(self) -> list:
        """ 关键信息列表 """
        colored_remaining = [
            colorize(s, "green") if s != "无" and s != "--" else s
            for s in self.remaining
        ]
        remaining_seats = "/".join(colored_remaining)
        if re.match("[GCD]", self.no):
            colored_no = colorize(self.no, self.no[0].lower())
        else:
            colored_no = colorize(self.no, "o")
        return [
            colored_no,
            self.start_time,
            self._fs,
            self._ts,
            self.end_time,
            remaining_seats,
            self.duration,
        ]

    def update_stations(self):
        """
            更新train对象中沿途车站的信息
        :param train: Train对象
        :param kwargs:
        :return: 无返回
        """
        glovar = Glovar()
        req = Request()
        s = req.get_session()

        # 准备请求参数
        params = {
            "train_no": self.full_no,
            "from_station_telecode": self.fs_code,
            "to_station_telecode": self.ts_code,
            "depart_date": glovar.date,
        }
        # 发送请求
        r = s.get(
            "https://kyfw.12306.cn/otn/czxx/queryByTrainNo", params=params, timeout=10
        )
        j = r.json()
        for item in j["data"]["data"]:
            if not item["isEnabled"]:
                continue
            station_code = station_to_code(item["station_name"])
            self.stations.append(
                (station_code, item["station_name"], item["arrive_time"])
            )


class TrainTable:
    """
    搜索结果列表
    """

    def __init__(self):
        self.trains_list = []

    def echo(self):
        """
        对外调用的方法，用来打印查询结果
        """
        tb = pt.PrettyTable()
        tb.field_names = ["车次", "发车", "出发站", "到达站", "到达", "余票", "历时"]
        self.cleanup()
        for train in self.trains_list:
            tb.add_row(train.row)
        print(tb)

    def cleanup(self):
        """ 处理trains_list，排序和删除无效数据 """
        glovar = Glovar()
        t_list = []
        if glovar.remaining:
            for train in self.trains_list:
                # print(train.no, train.remaining, train.has_remaining)
                if train.has_remaining:
                    t_list.append(train)
        else:
            t_list = self.trains_list
        self.trains_list = sorted(t_list)

    def update_trains(self, **kwargs):
        """
            对外调用的方法，查询12306列车信息，把结果更新到trains_list中
        :param kwargs:
        """
        glovar = Glovar()

        fs = kwargs.get("fs", "")
        ts = kwargs.get("ts", "")
        if fs and ts:
            # 优先处理fs和ts
            fs_code = station_to_code(fs)
            ts_code = station_to_code(ts)
        else:
            # 如果没有则再处理fs_code和ts_code
            fs_code = kwargs.get("fs_code", glovar.fs_code)
            ts_code = kwargs.get("ts_code", glovar.ts_code)

        date = kwargs.get("date", glovar.date)
        no_list = kwargs.get("no_list", glovar.no_list)

        if kwargs.get("zmode", glovar.zmode):
            self.trains_list = self._query_trains_zmode(fs_code, ts_code, date, no_list)
        else:
            self.trains_list = self._query_trains_basic(fs_code, ts_code, date, no_list)

    def _query_trains_basic(self, fs_code, ts_code, date, no_list) -> list:
        """
            普通查询方法，根据出发地和目的地编码、日期和限制车次，返回trains列表
            仅被内部调用，调用前处理好参数
        :param fs_code: 出发地编码
        :param ts_code: 目的地编码
        :param date: 日期
        :param no_list: 选择车次列表
        :return: 返回搜索结果列表，每一项是一个Train对象
        """
        glovar = Glovar()
        req = Request()
        s = req.get_session()
        # 准备请求参数
        params = {
            "leftTicketDTO.train_date": date,
            "leftTicketDTO.from_station": fs_code,
            "leftTicketDTO.to_station": ts_code,
            "purpose_codes": "ADULT",
        }
        try:
            # 发送请求
            print(params)
            r = s.get(
                QUERY_URL, params=params, timeout=10
            )
            j = r.json()
            raws = j["data"]["result"]
            # 处理返回结果
            train_list = []
            for raw in raws:
                fields = raw.split("|")
                # 如果限制了车次，且搜索车次不在目标车次中则丢弃
                if no_list and fields[3] not in no_list:
                    continue
                # 如果限制了种类
                if glovar.gcd:
                    # 只看高铁动车城际
                    if fields[3][0] not in "GCD":
                        continue
                elif glovar.ktz:
                    # 只看普通列车
                    if fields[3][0] in "GCD":
                        continue

                train = Train()
                train.full_no = fields[2]  # 编号全称
                train.no = fields[3]  # 编号简称
                train.fs_code = fields[6]
                train.ts_code = fields[7]
                train.start_time = fields[8]
                train.end_time = fields[9]
                train.duration = fields[10]
                for i in glovar.seats_idx_list:
                    train.remaining.append(fields[i] or "--")
                train_list.append(train)
        except Exception as e:
            print(e)
            print(r.text)
            print(s.headers)
            train_list = []
        return train_list

    def _query_trains_zmode(self, fs_code, ts_code, date, no_list) -> list:
        """
            高级查询模式，会查询从出发站到沿途所有站的车次情况
            仅被内部调用，调用前处理好参数
        :param fs_code: 出发地编码
        :param ts_code: 目的地编码
        :param date: 日期
        :param no_list: 限制车次
        :return: Train对象列表
        """
        # stations_set = set()
        stations_dict = {}
        trains_list = self._query_trains_basic(fs_code, ts_code, date, no_list)
        try:
            for train in trains_list:
                train.update_stations()
                for station in train.stations[1:-1]:  # 除了出发站和目标站本身
                    # stations_set.add(station[0])
                    if station[0] in stations_dict:
                        stations_dict[station[0]].append(train.no)
                    else:
                        stations_dict[station[0]] = [train.no]
            for station, trains in stations_dict.items():
                time.sleep(1)
                trains_list += self._query_trains_basic(fs_code, station, date, trains)
        except Exception as e:
            print(e)
        finally:
            return trains_list



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
@author: HJK
@file: __version__.py
@time: 2019-02-08
"""

__title__ = "x12306"
__description__ = "12306查票助手，一键查询沿途所有站点，在买不到票的时候，先上车后补票，暂不支持抢票"
__url__ = "https://github.com/0xHJK/x12306"
__version__ = "0.2.1"
__author__ = "HJK"
__author_email__ = "HJKdev@gmail.com"
__license__ = "MIT License"
__copyright__ = "Copyright 2024 HJK"



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
@author: HJK
@file: glovar.py
@time: 2019-02-11
"""

import sys
import datetime
from os import path

from .config import Options

# URL和JSESSIONID经常会变化，需要修改
# 12306查询URL，从浏览器抓包可得，最后的字母可能是A-Z
QUERY_URL = "https://kyfw.12306.cn/otn/leftTicket/queryE"
# JSESSIONID
JSESSIONID = "EC32F522D4074D3FAC9783DDDBFF0CFA"

# 大写配置无特殊情况不建议修改
FAKE_HEADERS = {
    "Accept": "*/*",
    "Accept-Encoding": "gzip, deflate, br",
    "Accept-Language": "zh-CN,zh;q=0.9,en-US;q=0.8,en;q=0.7,ja;q=0.6",
    "Cache-Control": "no-cache",
    "Connection": "keep-alive",
    "Host": "kyfw.12306.cn",
    "Referer": "https://kyfw.12306.cn/otn/leftTicket/init",
    "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_14_2) AppleWebKit/537.36 (KHTML, like Gecko) "
    "Chrome/72.0.3626.96 Safari/537.36",
    "X-Requested-With": "XMLHttpRequest",
    "Cookie": "JSESSIONID=" + JSESSIONID,
}

SEAT_TYPES = {
    "特等座": 25,
    "商务座": 32,
    "一等座": 31,
    "二等座": 30,
    "高级软卧": 21,
    "软卧": 23,
    "动卧": 33,
    "硬卧": 28,
    "软座": 24,
    "硬座": 29,
    "无座": 26,
    "其他": 22,
}

class Singleton(type):
    _instances = {}

    def __call__(cls, *args, **kwargs):
        if cls not in cls._instances:
            cls._instances[cls] = super(Singleton, cls).__call__(*args, **kwargs)
        return cls._instances[cls]


class Glovar(metaclass=Singleton):
    """
    全局共享变量
    """

    def __init__(self):
        opts = Options()

        if not path.isfile(opts.stations_file):
            # 如果stations文件不存在则终止程序
            print(opts.stations_file)
            print("Stations file is not exsits.")
            sys.exit(-1)

        self.zmode = opts.zmode
        self.zzmode = opts.zzmode
        self.total_stations = []
        self.total_proxies = []
        self.seats_list = []
        self.seats_idx_list = []
        self.fs = opts.fs
        self.ts = opts.ts
        self.fs_code = ""
        self.ts_code = ""
        self.date = opts.date or datetime.date.today().strftime("%Y-%m-%d")
        self.no_list = opts.train_no.split()
        self.gcd = opts.gcd
        self.ktz = opts.ktz
        self.remaining = opts.remaining

        # 获得所有站点编码的名称对应信息
        stations = open(opts.stations_file, "r", encoding="utf-8").read()
        for station in stations.split("@"):
            if not station:
                continue
            t = station.split("|")
            # (编码, 站名）如（VAP，北京）
            self.total_stations.append((t[2], t[1]))

        # 获得所有代理信息
        if path.isfile(opts.proxies_file):
            # 如果代理列表文件存在
            self.total_proxies = open(
                opts.proxies_file, "r", encoding="utf-8"
            ).readlines()

        # 处理座位选择
        for seat in opts.seats.split():
            if seat not in SEAT_TYPES:
                continue
            self.seats_list.append(seat)
            self.seats_idx_list.append(SEAT_TYPES[seat])

        # 处理站名编码
        for station in self.total_stations:
            if self.fs == station[1]:
                self.fs_code = station[0]
            if self.ts == station[1]:
                self.ts_code = station[0]
            if self.fs_code and self.ts_code:
                break
        # self.fs_code = station_to_code(self.fs)
        # print("-----glovar init")
        # print(self.fs_code, self.ts_code)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
@author: HJK
@file: config.py
@time: 2019-02-08
"""
"""
    全局配置和共享变量
"""

import platform
from os import path


class Singleton(type):
    _instances = {}

    def __call__(cls, *args, **kwargs):
        if cls not in cls._instances:
            cls._instances[cls] = super(Singleton, cls).__call__(*args, **kwargs)
        # else:
        #     cls._instances[cls].__init__(*args, **kwargs)
        return cls._instances[cls]


class Options(metaclass=Singleton):
    """
    自定义选项
    """

    def __init__(self):
        # 从哪里出发 -f --from-station
        self.fs = ""
        # 到哪里去 -t --to-station
        self.ts = ""
        # 日期 -d --date （格式：2019-01-01）
        self.date = ""
        # 车次号 -n --train-no （格式：G1 G2 G3）
        self.train_no = ""
        # 座位 -s --seats （格式：一等座 二等座 无座）
        self.seats = "一等座 二等座 无座"
        # 高级模式 -z --zmode （查询始发站往后到沿途的站点是否有票）
        self.zmode = False
        # 终极模式 -zz --zzmode （查询沿途所有站点之间是否有票）
        self.zzmode = False
        # 只看高铁动车城际 -g --gcd
        self.gcd = False
        # 只看普通列车（非gcd）-k --ktz
        self.ktz = False
        # 只看有余票 -r --remaining
        self.remaining = False

        # dir = path.abspath(path.join(path.dirname(__file__), ".."))
        dir = path.dirname(__file__)
        # 代理文件 --proxies-file （从文件中随机读取代理）
        self.proxies_file = path.join(dir, "data", "proxies.txt")
        # 站点信息文件 --stations-file （从12306下载）
        self.stations_file = path.join(dir, "data", "stations.txt")
        # CDN文件 --cdn-file
        self.cdn_file = path.join(dir, "data", "cdn.txt")

    def __str__(self):
        """返回配置信息"""
        s1 = "出发地：%s | 目的地：%s | 日期：%s\n" % (
            self.highlight(self.fs),
            self.highlight(self.ts),
            self.highlight(self.date),
        )
        s2 = "限制车次：%s | 限制座位：%s\n" % (
            self.highlight(self.train_no or "无"),
            self.highlight(self.seats),
        )
        s3 = "只看高铁动车：%s | 只看普通列车：%s\n" % (
            self.highlight(self.gcd),
            self.highlight(self.ktz),
        )
        s4 = "高级模式：%s | 终极模式：%s | 只看有票：%s\n" % (
            self.highlight(self.zmode),
            self.highlight(self.zzmode),
            self.highlight(self.remaining),
        )
        return s1 + s2 + s3 + s4

    def highlight(self, s):
        if platform.system() == "Windows":
            return str(s)
        return "\033[33m" + str(s) + "\033[0m"



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
@author: HJK
@file: setup.py.py
@time: 2019-02-11
"""

import os
import sys
import setuptools

# 'setup.py publish' shortcut.
if sys.argv[-1] == "publish":
    os.system("rm -rf dist")
    os.system("python setup.py sdist bdist_wheel")
    os.system("twine upload dist/*")
    sys.exit()

here = os.path.abspath(os.path.dirname(__file__))
about = {}
with open(os.path.join(here, "x12306", "__version__.py"), "r", encoding="utf-8") as f:
    exec(f.read(), about)

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name=about["__title__"],
    version=about["__version__"],
    description=about["__description__"],
    author=about["__author__"],
    author_email=about["__author_email__"],
    url=about["__url__"],
    license=about["__license__"],
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),
    # package_data={'x12306': ['data']},
    include_package_data=True,
    test_suite="tests",
    entry_points={
        "console_scripts": [
            "x12306 = x12306.__init__:main",
        ],
    },
    install_requires=[
        "requests",
        "click",
        "prettytable",
    ],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "Intended Audience :: End Users/Desktop",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Topic :: Internet",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Utilities",
    ],
)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
@author: HJK
@file: utils.py
@time: 2019-02-08
"""

import platform
from .glovar import Glovar


def station_to_code(name):
    for s in Glovar().total_stations:
        if name == s[1]:
            return s[0]


def code_to_station(code):
    for s in Glovar().total_stations:
        if code == s[0]:
            return s[1]


def colorize(s, color):
    colors = {
        "red": "\033[31m",
        "green": "\033[32m",
        "yellow": "\033[33m",
        "blue": "\033[34m",
        "pink": "\033[35m",
        "cyan": "\033[36m",
        # 高铁
        "g": "\033[91m",
        # 普通列车
        "o": "\033[92m",
        # 动车
        "d": "\033[93m",
        # 城际
        "c": "\033[93m",
        # "": "\033[94m",
        # "": "\033[95m",
        # "": "\033[96m",
    }
    if color not in colors or platform.system() == "Windows":
        return str(s)
    return colors[color] + str(s) + "\033[0m"



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
@author: HJK
@file: request.py
@time: 2019-02-08
"""

import random
import requests

from .glovar import Glovar, FAKE_HEADERS


class Request:
    """
    封装requests
    """

    def get_available_proxy(self) -> str:
        """
            获得一个可用的代理
        :return: 返回一个可用代理或空字符串
        """
        glovar = Glovar()
        if not glovar.total_proxies:
            # 如果没有可用代理则返回空字符串
            return ""

        proxy = random.choices(glovar.total_proxies)
        s = requests.Session()
        s.headers.update(FAKE_HEADERS)
        s.proxies.update({"http": proxy, "https": proxy})

        try:
            r = s.get("https://kyfw.12306.cn/otn/leftTicket/init", timeout=5)
            if r.status_code != requests.codes.ok or len(r.content) < 100:
                # 状态码不成功或内容长度太短也判定为失败
                raise RuntimeError
            else:
                return proxy
        except Exception:
            # 如果请求失败则从代理列表中删除
            glovar.total_proxies.remove(proxy)
            # 递归请求
            return self.get_available_proxy()

    def get_session(self):
        """ 获得一个session对象 """
        s = requests.Session()
        s.headers.update(FAKE_HEADERS)
        proxy = self.get_available_proxy()
        if proxy:
            s.proxies.update({"http": proxy, "https": proxy})
        return s
